/*
    addresstag -> '<address>' addresstuff '</address>'
  addressstuff -> nametag locationtag
       nametag -> '<name>' namestuff '</name>'
     namestuff -> firstnametag lastnametag [companytag]
                | lastnametag [firstnametag] [companytag]
                | companytag
  firstnametag -> '<first>' stringdata '</first>'
   lastnametag -> '<last>' stringdata '</last>'
    companytag -> '<company>' stringdata '</company>'
   locationtag -> '<location>' locationstuff '</location>'
 locationstuff -> numbertag streettag citytag statetag ziptag
     numbertag -> '<number>' stringdata '</number>'
     streettag -> '<street>' stringdata '</street>'
       citytag -> '<city>' stringdata '</city>'
      statetag -> '<state>' stringdata '</state>'
        ziptag -> '<zip>' stringdata '</zip>'
    stringdata -> any string that does not contain the characters '<' or '>'
*/

public class Parser
{

   public static void xml2obj(String addressXML, Address addressObj) throws Exception
   {
      Tokenizer2 tokens = new Tokenizer2(addressXML);

      //System.out.println( addressXML );  // for debugging
      //System.out.println( tokens );      // for debugging

      getAddressTag(tokens, addressObj);

   }//xml2obj

   public static void getAddressTag(Tokenizer2 tokens, Address addressObj) throws Exception
   {
      String token = tokens.nextToken();
      if ( !token.equals("address") )
         throw new Exception("Can't parse " + token);
      getAddressStuff(tokens, addressObj);
      token = tokens.nextToken();
      if ( !token.equals("/address") )
         throw new Exception("Can't parse " + token);
   }//getAddress

   public static void getAddressStuff(Tokenizer2 tokens, Address addressObj) throws Exception
   {
      getNameTag(tokens, addressObj);
      getLocationTag(tokens, addressObj);
   }//getAddressStuff

   public static void getNameTag(Tokenizer2 tokens, Address addressObj) throws Exception
   {
      String token = tokens.nextToken();
      if ( !token.equals("name") )
         throw new Exception("Can't parse " + token);
      getNameStuff(tokens, addressObj);
      token = tokens.nextToken();
      if ( !token.equals("/name") )
         throw new Exception("Can't parse " + token);
   }//getNameTag


   public static void getNameStuff(Tokenizer2 tokens, Address addressObj) throws Exception
   {
      String token = tokens.peekToken();
      if ( token.equals("first") )
      {
         getFirstnameTag(tokens, addressObj);
         getLastnameTag(tokens, addressObj);
         token = tokens.peekToken();
         if ( token.equals("company") )
         {
            getCompanyTag(tokens, addressObj);
         }
      }
      else if ( token.equals("last") )
      {
         getLastnameTag(tokens, addressObj);
         token = tokens.peekToken();
         if ( token.equals("first") )
         {
            getFirstnameTag(tokens, addressObj);
            token = tokens.peekToken();
            if ( token.equals("company") )
               getCompanyTag(tokens, addressObj);
         }
         else if ( token.equals("company") )
         {
            getCompanyTag(tokens, addressObj);
         }
      }
      else if ( token.equals("company") )
      {
         getCompanyTag(tokens, addressObj);
      }
      else
         throw new Exception("Can't parse " + token);

   }//getNameStuff


   public static void getFirstnameTag(Tokenizer2 tokens, Address addressObj) throws Exception
   {
      String token = tokens.nextToken();
      if ( !token.equals("first") )
         throw new Exception("Can't parse " + token);
      String first = tokens.nextToken();  // don't use this until we check for the closing tag
      token = tokens.nextToken();
      if ( !token.equals("/first") )
         throw new Exception("Can't parse " + token);
      addressObj.setFirst(first);
   }//getFirstnameTag


   public static void getLastnameTag(Tokenizer2 tokens, Address addressObj) throws Exception
   {
      String token = tokens.nextToken();
      if ( !token.equals("last") )
         throw new Exception("Can't parse " + token);
      String last = tokens.nextToken();  // don't use this until we check for the closing tag
      token = tokens.nextToken();
      if ( !token.equals("/last") )
         throw new Exception("Can't parse " + token);
      addressObj.setLast(last);
   }//getLastnameTag


   public static void getCompanyTag(Tokenizer2 tokens, Address addressObj) throws Exception
   {
      String token = tokens.nextToken();
      if ( !token.equals("company") )
         throw new Exception("Can't parse " + token);
      String company = tokens.nextToken();  // don't use this until we check for the closing tag
      token = tokens.nextToken();
      if ( !token.equals("/company") )
         throw new Exception("Can't parse " + token);
      addressObj.setCompany(company);
   }//getCompanyTag


   public static void getLocationTag(Tokenizer2 tokens, Address addressObj) throws Exception
   {
      String token = tokens.nextToken();
      if ( !token.equals("location") )
         throw new Exception("Can't parse " + token);
      getLocationStuff(tokens, addressObj);
      token = tokens.nextToken();
      if ( !token.equals("/location") )
         throw new Exception("Can't parse " + token);
   }//getLocationTag


   public static void getLocationStuff(Tokenizer2 tokens, Address addressObj) throws Exception
   {
      getNumberTag(tokens, addressObj);
      getStreetTag(tokens, addressObj);
      getCityTag(tokens, addressObj);
      getStateTag(tokens, addressObj);
      getZipTag(tokens, addressObj);
   }//getLocationStuff


   public static void getNumberTag(Tokenizer2 tokens, Address addressObj) throws Exception
   {
      String token = tokens.nextToken();
      if ( !token.equals("number") )
         throw new Exception("Can't parse " + token);
      String number = tokens.nextToken();  // don't use this until we check for the closing tag
      token = tokens.nextToken();
      if ( !token.equals("/number") )
         throw new Exception("Can't parse " + token);
      addressObj.setNumber(number);
   }//getNumberTag


   public static void getStreetTag(Tokenizer2 tokens, Address addressObj) throws Exception
   {
      String token = tokens.nextToken();
      if ( !token.equals("street") )
         throw new Exception("Can't parse " + token);
      String street = tokens.nextToken();  // don't use this until we check for the closing tag
      token = tokens.nextToken();
      if ( !token.equals("/street") )
         throw new Exception("Can't parse " + token);
      addressObj.setStreet(street);
   }//getStreetTag


   public static void getCityTag(Tokenizer2 tokens, Address addressObj) throws Exception
   {
      String token = tokens.nextToken();
      if ( !token.equals("city") )
         throw new Exception("Can't parse " + token);
      String city = tokens.nextToken();  // don't use this until we check for the closing tag
      token = tokens.nextToken();
      if ( !token.equals("/city") )
         throw new Exception("Can't parse " + token);
      addressObj.setCity(city);
   }//getCityTag


   public static void getStateTag(Tokenizer2 tokens, Address addressObj) throws Exception
   {
      String token = tokens.nextToken();
      if ( !token.equals("state") )
         throw new Exception("Can't parse " + token);
      String state = tokens.nextToken();  // don't use this until we check for the closing tag
      token = tokens.nextToken();
      if ( !token.equals("/state") )
         throw new Exception("Can't parse " + token);
      addressObj.setState(state);
   }//getStateTag


   public static void getZipTag(Tokenizer2 tokens, Address addressObj) throws Exception
   {
      String token = tokens.nextToken();
      if ( !token.equals("zip") )
         throw new Exception("Can't parse " + token);
      String zip = tokens.nextToken();  // don't use this until we check for the closing tag
      token = tokens.nextToken();
      if ( !token.equals("/zip") )
         throw new Exception("Can't parse " + token);
      addressObj.setZip(zip);
   }//getZipTag


//   public static void get(Tokenizer2 tokens, Address addressObj) throws Exception
//   {
//
//   }//



}//Parser