/*
   See http://www.json.org

   object -> '{' '}' | '{' members '}'

  members -> pair | pair ',' members

     pair -> string ':' value

    value -> string
           | number
           | object
           | array
           | true
           | false
           | null

    array -> '[' ']' | '[' elements ']'

 elements -> value | value ',' elements

*/

public class JSON2Parse
{

   public static void parseJSON(String jsonString) throws Exception
   {
      JSONTokenizer tokens = new JSONTokenizer(jsonString);

      getObject(tokens);

      if ( tokens.hasToken() )  // there shouldn't be any more tokens
         throw new Exception("\n  parseJSON() found unexpected input: "
                              + tokens.peekToken() );
   }//parseJSON()


   public static void getObject(JSONTokenizer tokens) throws Exception
   {
      String token = tokens.nextToken();
      if ( !token.equals("{") )
         throw new Exception("\n  getObject() expected '{' found '" + token + "'");

      if ( !tokens.peekToken().equals("}") )
         getMembers(tokens);

      token = tokens.nextToken();
      if ( !token.equals("}") )
         throw new Exception("\n  getObject() expected '}' found '" + token + "'");
   }//getObject()


   public static void getMembers(JSONTokenizer tokens) throws Exception
   {
      getPair(tokens);
      while ( tokens.peekToken().equals(",") )
      {
         tokens.nextToken(); // consume the ','
         getPair(tokens);
      }
   }//getMemebers()


   public static void getPair(JSONTokenizer tokens) throws Exception
   {
      String token = tokens.nextToken();

      token = tokens.nextToken();
      if ( !token.equals(":") )
         throw new Exception("\n  getPair() expected ':' found '" + token + "'");

      getValue(tokens);
   }//getPair()


   public static void getValue(JSONTokenizer tokens) throws Exception
   {
      String token = tokens.peekToken();
      if ( token.equals("{") )
      {
         getObject(tokens);
      }
      else if ( token.equals("[") )
      {
         getArray(tokens);
      }
      else if ( token.equals("\"") )
      {
         getString(tokens);
      }
      else if ( token.equals("true") )
      {
         tokens.nextToken();
      }
      else if ( token.equals("false") )
      {
         tokens.nextToken();
      }
      else if ( token.equals("null") )
      {
         tokens.nextToken();
      }
      else  // number
      {
         tokens.nextToken();
      }

   }//getValue()


   public static void getArray(JSONTokenizer tokens) throws Exception
   {
      String token = tokens.nextToken();
      if ( !token.equals("[") )
         throw new Exception("\n  getArray() expected '[' found '" + token + "'");

      if ( !tokens.peekToken().equals("]") )
         getElements(tokens);

      token = tokens.nextToken();
      if ( !token.equals("]") )
         throw new Exception("\n  getArray() expected ']' found '" + token + "'");
   }//getArray()


   public static void getElements(JSONTokenizer tokens) throws Exception
   {
      getValue(tokens);
      while ( tokens.peekToken().equals(",") )
      {
         tokens.nextToken(); // consume the ','
         getValue(tokens);
      }
   }//getElements()


   public static void getString(JSONTokenizer tokens) throws Exception
   {
      String token = tokens.nextToken();
      if ( !token.equals("\"") )
         throw new Exception("\n  getString() expected '\"' found '" + token + "'");

      tokens.nextToken();  // consume the String

      token = tokens.nextToken();
      if ( !token.equals("\"") )
         throw new Exception("\n  getString() expected '\"' found '" + token + "'");
   }//getString()

}//JSON2Parse
