/*
   See http://www.json.org

   object -> '{' '}' | '{' members '}'

  members -> pair | pair ',' members

     pair -> string ':' value

    value -> string
           | number
           | object
           | array
           | true
           | false
           | null

    array -> '[' ']' | '[' elements ']'

 elements -> value | value ',' elements

*/

public class JSON2ParseTree
{

   public static Tree parseJSON(String jsonString) throws Exception
   {
      JSONTokenizer tokens = new JSONTokenizer(jsonString);

      Tree result = getObject(tokens);

      if ( tokens.hasToken() )  // there shouldn't be any more tokens
         throw new Exception("\n  parseJSON() found unexpected input: "
                              + tokens.peekToken() );
      return result;
   }//parseJSON()


   public static Tree getObject(JSONTokenizer tokens) throws Exception
   {
      Tree result = new Tree("Object");

      String token = tokens.nextToken();
      if ( !token.equals("{") )
         throw new Exception("\n  getObject() expected '{' found '" + token + "'");
      result.addSubTree( new Tree(token) );

      if ( !tokens.peekToken().equals("}") )
         result.addSubTree( getMembers(tokens) );

      token = tokens.nextToken();
      if ( !token.equals("}") )
         throw new Exception("\n  getObject() expected '}' found '" + token + "'");
      result.addSubTree( new Tree(token) );

      return result;
   }//getObject()


   public static Tree getMembers(JSONTokenizer tokens) throws Exception
   {
      Tree result = new Tree("Members");

      result.addSubTree( getPair(tokens) );

      while ( tokens.peekToken().equals(",") )
      {
         result.addSubTree( new Tree(tokens.nextToken()) ); // consume the ','
         result.addSubTree( getPair(tokens) );
      }

      return result;
   }//getMemebers()


   public static Tree getPair(JSONTokenizer tokens) throws Exception
   {
      Tree result = new Tree("Pair");

      result.addSubTree( new Tree(tokens.nextToken()) );

      String token = tokens.nextToken();
      if ( !token.equals(":") )
         throw new Exception("\n  getPair() expected ':' found '" + token + "'");
      result.addSubTree( new Tree(token) );

      result.addSubTree( getValue(tokens) );

      return result;
   }//getPair()


   public static Tree getValue(JSONTokenizer tokens) throws Exception
   {
      Tree result = new Tree("Value");

      String token = tokens.peekToken();
      if ( token.equals("{") )
      {
         result.addSubTree( getObject(tokens) );
      }
      else if ( token.equals("[") )
      {
         result.addSubTree( getArray(tokens) );
      }
      else if ( token.equals("\"") )
      {
         result.addSubTree( getString(tokens) );
      }
      else if ( token.equals("true") )
      {
         result.addSubTree( new Tree(tokens.nextToken()) );
      }
      else if ( token.equals("false") )
      {
         result.addSubTree( new Tree(tokens.nextToken()) );
      }
      else if ( token.equals("null") )
      {
         result.addSubTree( new Tree(tokens.nextToken()) );
      }
      else  // number
      {
         result.addSubTree( new Tree(tokens.nextToken()) );
      }
      return result;
   }//getValue()


   public static Tree getArray(JSONTokenizer tokens) throws Exception
   {
      Tree result = new Tree("Array");

      String token = tokens.nextToken();
      if ( !token.equals("[") )
         throw new Exception("\n  getArray() expected '[' found '" + token + "'");
      result.addSubTree( new Tree(token) );

      if ( !tokens.peekToken().equals("]") )
         result.addSubTree( getElements(tokens) );

      token = tokens.nextToken();
      if ( !token.equals("]") )
         throw new Exception("\n  getArray() expected ']' found '" + token + "'");
      result.addSubTree( new Tree(token) );

      return result;
   }//getArray()


   public static Tree getElements(JSONTokenizer tokens) throws Exception
   {
      Tree result = new Tree("Elements");

      result.addSubTree( getValue(tokens) );
      while ( tokens.peekToken().equals(",") )
      {
         result.addSubTree( new Tree(tokens.nextToken()) ); // consume the ','
         result.addSubTree( getValue(tokens) );
      }
      return result;
   }//getElements()


   public static Tree getString(JSONTokenizer tokens) throws Exception
   {
      Tree result = new Tree("String");

      String token = tokens.nextToken();
      if ( !token.equals("\"") )
         throw new Exception("\n  getString() expected '\"' found '" + token + "'");
      result.addSubTree( new Tree("\\"+token) );

      result.addSubTree( new Tree(tokens.nextToken()) );  // consume the String

      token = tokens.nextToken();
      if ( !token.equals("\"") )
         throw new Exception("\n  getString() expected '\"' found '" + token + "'");
      result.addSubTree( new Tree("\\"+token) );

      return result;
   }//getString()

}//JSON2ParseTree
