/*
   This program creates "data structure" trees. These are similar to
   "expression trees" in that they are much more compact than the full
   parse tree, and, like an expression tree, a data structure tree is a
   complete description of the "structure" of the string that was parsed.
*/
import java.io.PrintWriter;
import java.io.File;
import java.lang.Runtime;

public class TestJSON2DataStructureTree
{
   public static void main(String[] args) throws Exception
   {
      String[] json =
      {
         "{}",

         "{ a1:[], a2:[] }",

         "{o1:{}, a1:[], s1:3}",

         "{o1:{x:1, y:2, z:3}, a1:[123,[{x:1}, {y:2}, {z:3}]], s1:\"pears\"}"
      };

      for (int i = 0; i < json.length; i++)
      {
         System.out.println( json[i] );
         Tree dataStructureTree = JSON2DataStructureTree.parseJSON( json[i] );
         System.out.println( dataStructureTree+"\n" );

         // Create dot and png files from the two trees.
         try
         {
            // Create the dot file.
            PrintWriter out = new PrintWriter(
                              new File("JSON-"+(i+1)+"-DataStructureTree.dot") );
            // Put dot commands into the dot file.
            out.println(  Tree2dot.tree2dot(dataStructureTree) + "\n" );
            out.close();

            // Create a Windows command lines for running dot.exe.
            String[] cmd = {"dot.exe",
                            "-Tpng",
                            "JSON-"+(i+1)+"-DataStructureTree.dot",
                            "-o",
                            "JSON-"+(i+1)+"-DataStructureTree.png"};
            // Execute the command line.
            Runtime.getRuntime().exec(cmd);
         }
         catch (Exception e)
         {
            System.out.println( e );
         }
      }
   }//main()
}//TestJSON2DataStructureTree